# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import logging as log
import os

from .opt_in_checker import OptInChecker


def save_cid_to_file(file_name: str, cid: str):
    """
    Save the client ID to the specified file
    """
    try:
        # create directories recursively first
        os.makedirs(os.path.dirname(file_name), exist_ok=True)

        with open(file_name, 'w') as file:
            file.write(cid)
    except Exception as e:
        return False
    return True


def get_cid(file_name: str, validator: [callable, None]):
    """
    Get existing Client ID.
    :param file_name: name of the file with the client ID
    :param validator: the function to validate the client ID
    :return: existing client ID file
    """
    full_path = os.path.join(get_cid_path(), file_name)
    cid = None
    if os.path.exists(full_path):
        with open(full_path, 'r') as file:
            cid = file.readline().strip()
        if cid is not None and (validator is not None and not validator(cid)):
            cid = None
    return cid


def get_or_generate_cid(file_name: str, generator: callable, validator: [callable, None], old_name=None):
    """
    Get existing Client ID or generate a new one.
    :param file_name: name of the file with the client ID
    :param generator: the function to generate the client ID
    :param validator: the function to validate the client ID
    :param old_name: legacy name of the file with the client ID
    :return: existing or a new client ID file
    """
    cid = get_cid(file_name, validator)
    if cid is not None:
        return cid
    if old_name is not None:
        cid = get_cid(old_name, validator)
    full_path = os.path.join(get_cid_path(), file_name)
    if cid is None:
        cid = generator()
    save_cid_to_file(full_path, cid)
    return cid


def get_cid_path():
    """
    Returns a directory with the OpenVINO randomly generated client ID file.

    :return: the directory with the client ID file
    """
    return os.path.join(OptInChecker.consent_file_base_dir(), OptInChecker.consent_file_subdirectory())


def remove_cid_file(file_name: str):
    """
    Removes client ID file.
    :param file_name: name of the file with the client ID
    :return: None
    """
    cid_file = os.path.join(get_cid_path(), file_name)
    if os.path.exists(cid_file):
        if not os.access(cid_file, os.W_OK):
            log.warning("Failed to remove client ID file {}.".format(cid_file))
            return
        os.remove(cid_file)



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import logging as log
import os
import sys
from enum import Enum

from .backend.backend import BackendRegistry
from .utils.sender import TelemetrySender
from .utils.opt_in_checker import OptInChecker, ConsentCheckResult, DialogResult
from .utils.stats_processor import StatsProcessor


class OptInStatus(Enum):
    ACCEPTED = "accepted"
    DECLINED = "declined"
    UNDEFINED = "undefined"


class SingletonMetaClass(type):
    def __init__(self, cls_name, super_classes, dic):
        self.__single_instance = None
        super().__init__(cls_name, super_classes, dic)

    def __call__(cls, *args, **kwargs):
        if cls.__single_instance is None:
            cls.__single_instance = super(SingletonMetaClass, cls).__call__(*args, **kwargs)
        return cls.__single_instance


class Telemetry(metaclass=SingletonMetaClass):
    """
    The main class to send telemetry data. It uses singleton pattern. The instance should be initialized with the
    application name, version and tracking id just once. Later the instance can be created without parameters.
    Args:
        :param app_name: The name of the application.
        :param app_version: The version of the application.
        :param tid: The ID of telemetry base.
        :param backend: Telemetry backend name.
        :param enable_opt_in_dialog: boolean flag to turn on or turn off opt-in dialog.
        If enable_opt_in_dialog=True opt-in dialog is shown during first usage of openvino tools,
        no telemetry is sent until user accepts telemetry with dialog.
        If enable_opt_in_dialog=False, telemetry is sent without opt-in dialog, unless user explicitly turned it off
        with opt_in_out script.
        :param disable_in_ci: Turn off telemetry for CI jobs.
    """

    def __init__(self, app_name: str = None, app_version: str = None, tid: str = None,
                 backend: [str, None] = 'ga', enable_opt_in_dialog=True, disable_in_ci=False):
        # The case when instance is already configured
        if app_name is None:
            if not hasattr(self, 'sender') or self.sender is None:
                raise RuntimeError('The first instantiation of the Telemetry should be done with the '
                                   'application name, version and TID.')
            return

        self.init(app_name, app_version, tid, backend, enable_opt_in_dialog, disable_in_ci, True)

    def init(self, app_name: str = None, app_version: str = None, tid: str = None,
             backend: [str, None] = 'ga', enable_opt_in_dialog=True, disable_in_ci=False, increment_stats=False):
        opt_in_checker = OptInChecker()
        opt_in_check_result = opt_in_checker.check(enable_opt_in_dialog, disable_in_ci)
        if enable_opt_in_dialog:
            self.consent = opt_in_check_result == ConsentCheckResult.ACCEPTED
        else:
            self.consent = opt_in_check_result == ConsentCheckResult.ACCEPTED or \
                           opt_in_check_result == ConsentCheckResult.NO_FILE

        if tid is None:
            log.warning("Telemetry will not be sent as TID is not specified.")

        self.tid = tid
        self.backend = BackendRegistry.get_backend(backend)(self.tid, app_name, app_version)
        self.sender = TelemetrySender()

        if self.consent and not self.backend.cid_file_initialized():
            self.backend.generate_new_cid_file()

        if self.consent:
            data = self.get_stats(increment_stats)
            if data is not None and isinstance(data, dict):
                self.backend.set_stats(data)

        if not enable_opt_in_dialog and self.consent:
            # Try to create directory for client ID if it does not exist
            if not opt_in_checker.create_or_check_consent_dir():
                log.warning("Could not create directory for storing client ID. No data will be sent.")
                return

            # Generate client ID if it does not exist
            if not self.backend.cid_file_initialized():
                self.backend.generate_new_cid_file()
            return

        # Consent file may be absent, for example, during the first run of Openvino tool.
        # In this case we trigger opt-in dialog that asks user permission for sending telemetry.
        if opt_in_check_result == ConsentCheckResult.NO_FILE:
            if opt_in_checker.create_or_check_consent_dir():
                answer = ConsentCheckResult.DECLINED

                # check if it is openvino tool
                if not self.check_by_cmd_line_if_dialog_needed():
                    return

                # create openvino_telemetry file if possible with "0" value
                if not opt_in_checker.update_result(answer):
                    return
                try:
                    # run opt-in dialog
                    answer = opt_in_checker.opt_in_dialog()
                    if answer == DialogResult.ACCEPTED:
                        # If the dialog result is "accepted" we generate new client ID file and update openvino_telemetry
                        # file with "1" value. Telemetry data will be collected in this case.
                        self.consent = True
                        self.backend.generate_new_cid_file()
                        self.send_opt_in_event(OptInStatus.ACCEPTED)

                        # Here we send telemetry with "accepted" dialog result
                        opt_in_checker.update_result(ConsentCheckResult.ACCEPTED)
                    elif answer == DialogResult.TIMEOUT_REACHED:
                        # If timer was reached and we have no response from user, we should not send
                        # any data except for dialog result.
                        # At this point we have already created openvino_telemetry file with "0" value,
                        # which means that no telemetry data will be collected in further functions.
                        # As openvino_telemetry file exists on the system the dialog won't be shown again.

                        # Here we send telemetry with "timer reached" dialog result
                        self.send_opt_in_event(OptInStatus.UNDEFINED, force_send=True)
                    else:
                        # If the dialog result is "declined" we should not send any data except for dialog result.
                        # At this point we have already created openvino_telemetry file with "0" value,
                        # which means that no telemetry data will be collected in further functions.
                        # As openvino_telemetry file exists on the system the dialog won't be shown again.

                        # Here we send telemetry with "declined" dialog result
                        self.send_opt_in_event(OptInStatus.DECLINED, force_send=True)
                except KeyboardInterrupt:
                    pass

    def check_by_cmd_line_if_dialog_needed(self):
        scripts_to_run_dialog = [
            os.path.join("openvino", "tools", "mo", "main"),
            os.path.join("openvino", "tools", "ovc", "main"),
            "ovc",
            "mo",
            "pot",
            "omz_downloader",
            "omz_converter",
            "omz_data_downloader",
            "omz_info_dumper",
            "omz_quantizer",
            "accuracy_check"
        ]
        extensions = [".py", ".exe", ""]
        args = sys.argv
        if len(args) == 0:
            return False
        script_name = args[0]

        for script_to_run_dialog in scripts_to_run_dialog:
            for ext in extensions:
                script_to_check = script_to_run_dialog + ext
                if script_name == script_to_check or script_name.endswith(os.sep + script_to_check):
                    return True
        return False

    def force_shutdown(self, timeout: float = 1.0):
        """
        Stops currently running threads which may be hanging because of no Internet connection.

        :param timeout: maximum timeout time
        :return: None
        """
        self.sender.force_shutdown(timeout)

    def send_event(self, event_category: str, event_action: str, event_label: str, event_value: int = 1,
                   app_name=None, app_version=None, force_send=False, **kwargs):
        """
        Send single event.

        :param event_category: category of the event
        :param event_action: action of the event
        :param event_label: the label associated with the action
        :param event_value: the integer value corresponding to this label
        :param app_name: application name
        :param app_version: application version
        :param force_send: forces to send event ignoring the consent value
        :param kwargs: additional parameters
        :return: None
        """
        if self.consent or force_send:
            self.sender.send(self.backend, self.backend.build_event_message(event_category, event_action, event_label,
                                                                            event_value, app_name, app_version,
                                                                            **kwargs))

    def start_session(self, category: str, **kwargs):
        """
        Sends a message about starting of a new session.

        :param kwargs: additional parameters
        :param category: the application code
        :return: None
        """
        if self.consent:
            self.sender.send(self.backend, self.backend.build_session_start_message(category, **kwargs))

    def end_session(self, category: str, **kwargs):
        """
        Sends a message about ending of the current session.

        :param kwargs: additional parameters
        :param category: the application code
        :return: None
        """
        if self.consent:
            self.sender.send(self.backend, self.backend.build_session_end_message(category, **kwargs))

    def send_error(self, category: str, error_msg: str, **kwargs):
        if self.consent:
            self.sender.send(self.backend, self.backend.build_error_message(category, error_msg, **kwargs))

    def send_stack_trace(self, category: str, stack_trace: str, **kwargs):
        if self.consent:
            self.sender.send(self.backend, self.backend.build_stack_trace_message(category, stack_trace, **kwargs))

    @staticmethod
    def _update_opt_in_status(tid: str, new_opt_in_status: bool):
        """
        Updates opt-in status.

        :param tid: ID of telemetry base.
        :param new_opt_in_status: new opt-in status.
        :return: None
        """
        app_name = 'opt_in_out'
        app_version = Telemetry.get_version()
        opt_in_checker = OptInChecker()
        # If enable_opt_in_dialog=True is set opt_in_checker.check() makes extra checks,
        # if it is the main process or a notebook opt_in_checker.check() returns ConsentCheckResult.DECLINED.
        # It is needed only in init() method to prevent opt-in dialog for these cases.
        # Here these checks are not needed, because _update_opt_in_status() is executed only from opt_in_out
        # which does not trigger opt-in dialog in any case.
        opt_in_check = opt_in_checker.check(enable_opt_in_dialog=False)

        prev_status = OptInStatus.UNDEFINED
        if opt_in_check == ConsentCheckResult.DECLINED:
            prev_status = OptInStatus.DECLINED
        elif opt_in_check == ConsentCheckResult.ACCEPTED:
            prev_status = OptInStatus.ACCEPTED

        if new_opt_in_status:
            updated = opt_in_checker.update_result(ConsentCheckResult.ACCEPTED)
        else:
            updated = opt_in_checker.update_result(ConsentCheckResult.DECLINED)
        if not updated:
            print("Could not update the consent file. No telemetry will be sent.")
            return

        telemetry = Telemetry(tid=tid, app_name=app_name, app_version=app_version, backend='ga4')

        # In order to prevent sending of duplicate events, after multiple run of opt_in_out --opt_in/--opt_out
        # we send opt_in event only if consent value is changed
        if new_opt_in_status:
            telemetry.backend.generate_new_cid_file()
            if prev_status != OptInStatus.ACCEPTED:
                telemetry.send_opt_in_event(OptInStatus.ACCEPTED, prev_status)
            print("You have successfully opted in to send the telemetry data.")
        else:
            if prev_status != OptInStatus.DECLINED:
                telemetry.send_opt_in_event(OptInStatus.DECLINED, prev_status, force_send=True)
            telemetry.backend.remove_cid_file()
            from .utils.stats_processor import StatsProcessor
            StatsProcessor().remove_stats_file()
            print("You have successfully opted out to send the telemetry data.")

    def send_opt_in_event(self, new_state: OptInStatus, prev_state: OptInStatus = OptInStatus.UNDEFINED,
                          label: str = "", force_send=False):
        """
        Sends opt-in event.

        :param new_state: new opt-in status.
        :param prev_state: previous opt-in status.
        :param label: the label with the information of opt-in status change.
        :param force_send: forces to send event ignoring the consent value
        :return: None
        """
        if new_state == OptInStatus.UNDEFINED:
            self.send_event("opt_in", "timer_reached", label, force_send=force_send)
        else:
            label = "{{prev_state:{}, new_state: {}}}".format(prev_state.value, new_state.value)
            self.send_event("opt_in", new_state.value, label, force_send=force_send)

    def get_stats(self, update_usage_num: bool):
        stats = StatsProcessor()
        file_exists, data = stats.get_stats()
        if not file_exists:
            created = stats.create_new_stats_file()
            data = {}
            if not created:
                return None
        if "usage_count" in data:
            usage_count = data["usage_count"]
            if usage_count is None or not isinstance(usage_count, int) or usage_count <= 0:
                log.warning("Invalid usage count.")
                return data
            if usage_count < sys.maxsize and update_usage_num:
                usage_count += 1
        else:
            usage_count = 1
        data["usage_count"] = usage_count
        stats.update_stats(data)
        if usage_count == 1:
            data["usage_group"] = "first_usage"
        elif usage_count <= 100:
            data["usage_group"] = "1-100_usages"
        elif usage_count <= 1000:
            data["usage_group"] = "101-1000_usages"
        elif usage_count <= 5000:
            data["usage_group"] = "1001-5000_usages"
        else:
            data["usage_group"] = "5000+_usages"

        return data

    @staticmethod
    def opt_in(tid: str):
        """
        Enables sending anonymous telemetry data.

        :param tid: ID of telemetry base.
        :return: None
        """
        Telemetry._update_opt_in_status(tid, True)

    @staticmethod
    def opt_out(tid: str):
        """
        Disables sending anonymous telemetry data.

        :param tid: ID of telemetry base.
        :return: None
        """
        Telemetry._update_opt_in_status(tid, False)

    @staticmethod
    def get_version():
        """
        Returns version of telemetry library.
        """
        return '2025.2.0'



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import logging as log
import uuid
from urllib import request, parse

from .backend import TelemetryBackend
from ..utils.cid import get_or_generate_cid, remove_cid_file
from ..utils.message import Message, MessageType


class GABackend(TelemetryBackend):
    backend_url = 'https://www.google-analytics.com/collect'
    id = 'ga'
    cid_filename = 'openvino_ga_cid'

    def __init__(self, tid: str = None, app_name: str = None, app_version: str = None):
        super(GABackend, self).__init__(tid, app_name, app_version)
        self.tid = tid
        self.cid = None
        self.app_name = app_name
        self.app_version = app_version
        self.default_message_attrs = {
            'v': '1',  # API Version
            'tid': self.tid,
            'an': self.app_name,
            'av': self.app_version,
            'ua': 'Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14'  # dummy identifier of the browser
        }

    def send(self, message: Message):
        if self.cid is None:
            message.attrs['cid'] = str(uuid.uuid4())
        try:
            data = parse.urlencode(message.attrs).encode()

            if self.backend_url.lower().startswith('http'):
                req = request.Request(self.backend_url, data=data)
            else:
                log.info("Incorrect backend URL.")
                return

            request.urlopen(req) #nosec
        except Exception as err:
            pass  # nosec

    def build_event_message(self, event_category: str, event_action: str, event_label: str, event_value: int = 1,
                            **kwargs):
        data = self.default_message_attrs.copy()
        data.update({
            't': 'event',
            'ec': event_category,
            'ea': event_action,
            'el': event_label,
            'ev': event_value,
        })
        return Message(MessageType.EVENT, data)

    def build_session_start_message(self, category: str, **kwargs):
        data = self.default_message_attrs.copy()
        data.update({
            'sc': 'start',
            't': 'event',
            'ec': category,
            'ea': 'session',
            'el': 'start',
            'ev': 1,
        })
        return Message(MessageType.SESSION_START, data)

    def build_session_end_message(self, category: str, **kwargs):
        data = self.default_message_attrs.copy()
        data.update({
            'sc': 'end',
            't': 'event',
            'ec': category,
            'ea': 'session',
            'el': 'end',
            'ev': 1,
        })
        return Message(MessageType.SESSION_END, data)

    def build_error_message(self, category: str, error_msg: str, **kwargs):
        data = self.default_message_attrs.copy()
        data.update({
            't': 'event',
            'ec': category,
            'ea': 'error',
            'el': error_msg,
            'ev': 1,
        })
        return Message(MessageType.ERROR, data)

    def build_stack_trace_message(self, category: str, error_msg: str, **kwargs):
        data = self.default_message_attrs.copy()
        data.update({
            't': 'event',
            'ec': category,
            'ea': 'stack_trace',
            'el': error_msg,
            'ev': 1,
        })
        return Message(MessageType.STACK_TRACE, data)

    def remove_cid_file(self):
        self.cid = None
        self.default_message_attrs['cid'] = None
        remove_cid_file(self.cid_filename)

    def generate_new_cid_file(self):
        self.cid = get_or_generate_cid(self.cid_filename, lambda: str(uuid.uuid4()), is_valid_cid)
        self.default_message_attrs['cid'] = self.cid

    def cid_file_initialized(self):
        return self.cid is not None

    def set_stats(self, data: dict):
        pass


def is_valid_cid(cid: str):
    try:
        uuid.UUID(cid, version=4)
    except ValueError:
        return False
    return True



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import logging as log
import threading
from concurrent import futures
from time import sleep

from ..backend.backend import TelemetryBackend
from ..utils.message import Message

MAX_QUEUE_SIZE = 1000


class TelemetrySender:
    def __init__(self, max_workers=None):
        self.executor = futures.ThreadPoolExecutor(max_workers=max_workers)
        self.queue_size = 0
        self._lock = threading.Lock()

    def send(self, backend: TelemetryBackend, message: Message):
        def _future_callback(future):
            with self._lock:
                self.queue_size -= 1

        free_space = False
        with self._lock:
            if self.queue_size < MAX_QUEUE_SIZE:
                free_space = True
                self.queue_size += 1
            else:
                pass  # dropping a message because the queue is full
        # to avoid dead lock we should not add callback inside the "with self._lock" block because it will be executed
        # immediately if the fut is available
        if free_space:
            try:
                fut = self.executor.submit(backend.send, message)
                fut.add_done_callback(_future_callback)
            except Exception as err:
                pass  # nosec

    def force_shutdown(self, timeout: float):
        """
        Forces all threads to be stopped after timeout. The "shutdown" method of the ThreadPoolExecutor removes only not
        yet scheduled threads and keep running the existing one. In order to stop the already running use some low-level
        attribute. The operation with low-level attributes is wrapped with the try/except to avoid potential crash if
        these attributes will removed or renamed.

        :param timeout: timeout to wait before the shutdown
        :return: None
        """
        need_sleep = False
        with self._lock:
            if self.queue_size > 0:
                need_sleep = True
        if need_sleep:
            sleep(timeout)
        self.executor.shutdown(wait=False, cancel_futures=True)

        try:
            self.executor._threads.clear()
        except Exception as err:
            pass  # nosec



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class MessageType(Enum):
    EVENT = 0
    ERROR = 1
    STACK_TRACE = 2
    SESSION_START = 3
    SESSION_END = 4


class Message:
    def __init__(self, type: MessageType, attrs: dict):
        self.type = type
        self.attrs = attrs.copy()



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from .opt_in_checker import OptInChecker
import logging as log
import os
import json


class StatsProcessor:
    """
    The class is used for storing of additional statistics, that need be stored on the local system.
    For example, number of usages of OpenVino.
    The class is used by main Telemetry class, for example:

    stats_processor = StatsProcessor()
    created = stats_processor.create_new_stats_file()

    # store statistics
    updated = stats_processor.update_stats({"usage_count": 1})

    # read statistics
    read_status, stats = stats_processor.get_stats()

    # remove statistics file
    stats_processor.remove_stats_file()
    """
    def __init__(self):
        self.opt_in_checker = OptInChecker()

    def stats_file(self):
        """
        Returns the statistics file path.
        """
        return os.path.join(self.opt_in_checker.consent_file_base_dir(), self.opt_in_checker.consent_file_subdirectory(), "stats")

    def create_new_stats_file(self):
        """
        Creates a new statistics file.
        :return: True if the file is created successfully, otherwise False
        """
        if not self.opt_in_checker.create_or_check_consent_dir():
            return False
        try:
            open(self.stats_file(), 'w').close()
        except Exception:
            return False
        return True

    def update_stats(self, stats: dict):
        """
        Updates the statistics in the statistics file.
        :param stats: the dictionary with statistics.
        :return: False if the statistics file is not writable, otherwise True
        """
        if self.opt_in_checker.consent_file_base_dir() is None or self.opt_in_checker.consent_file_subdirectory() is None:
            return False
        if not os.path.exists(self.stats_file()):
            if not self.create_new_stats_file():
                return False
        if not os.access(self.stats_file(), os.W_OK):
            return False
        try:
            str_data = json.dumps(stats, indent=4)
            with open(self.stats_file(), 'w') as file:
                file.write(str_data)
        except Exception:
            return False
        return True

    def get_stats(self):
        """
        Gets information from statistics file.
        :return: the tuple, where the first element is True if the file is read successfully, otherwise False
        and the second element is the content of the statistics file.
        """
        if not os.access(self.stats_file(), os.R_OK):
            return False, {}
        try:
            with open(self.stats_file(), 'r') as file:
                data = json.load(file)
        except Exception:
            return False, {}
        return True, data

    def remove_stats_file(self):
        """
        Removes statistics file.
        :return: None
        """
        stats_file = self.stats_file()
        if os.path.exists(stats_file):
            if not os.access(stats_file, os.W_OK):
                return
            os.remove(stats_file)



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import json
import uuid
import logging as log

from copy import copy
from urllib import request
import os

from .backend import TelemetryBackend
from ..utils.cid import get_or_generate_cid, remove_cid_file
from ..utils.params import telemetry_params
from platform import system


def _send_func(request_data):
    try:
        request.urlopen(request_data)  # nosec
    except Exception as err:
        pass  # nosec


def is_docker():
    def file_has_text(text, filename):
        try:
            with open(filename, encoding='utf-8') as lines:
                for line in lines:
                    if text in line:
                        return True
            return False
        except:
            return False

    return os.path.exists('/.dockerenv') or \
           file_has_text('docker', '/proc/self/cgroup') or \
           file_has_text('docker', '/proc/self/mountinfo')


class GA4Backend(TelemetryBackend):
    id = 'ga4'
    cid_filename = 'openvino_ga_cid'
    old_cid_filename = 'openvino_ga_uid'
    timeout = 3.0

    def __init__(self, tid: str = None, app_name: str = None, app_version: str = None):
        super(GA4Backend, self).__init__(tid, app_name, app_version)
        self.measurement_id = tid
        self.app_name = app_name
        self.app_version = app_version
        self.session_id = None
        self.cid = None
        self.backend_url = "https://www.google-analytics.com/mp/collect?measurement_id={}&api_secret={}".format(
            self.measurement_id, telemetry_params["api_key"])
        self.default_message_attrs = {
            'app_name': self.app_name,
            'app_version': self.app_version,
            'os': system(),
        }
        self.stats = {}

    def send(self, message: dict):
        if message is None:
            return
        try:
            data = json.dumps(message).encode()

            if self.backend_url.lower().startswith('http'):
                req = request.Request(self.backend_url, data=data)
            else:
                log.info("Incorrect backend URL.")
                return
            if system() == 'Windows':
                _send_func(req)
            else:
                # request.urlopen() may hang on Linux if there's no internet connection,
                # so we need to run it in a subprocess and terminate after timeout.

                # Usage of subprocesses on Windows cause unexpected behavior, when script
                # executes multiple times during subprocess initializing. For this reason
                # subprocess are not recommended on Windows.
                import multiprocessing
                process = multiprocessing.Process(target=_send_func, args=(req,))
                process.daemon = True
                process.start()

                process.join(self.timeout)
                if process.is_alive():
                    process.terminate()

        except Exception as err:
            pass  # nosec

    def build_event_message(self, event_category: str, event_action: str, event_label: str, event_value: int = 1,
                            app_name=None, app_version=None,
                            **kwargs):
        client_id = self.cid
        if client_id is None:
            client_id = "0"
        if self.session_id is None:
            self.generate_new_session_id()

        default_args = copy(self.default_message_attrs)
        default_args['docker'] = 'False'
        if app_name is not None:
            default_args['app_name'] = app_name
        if app_version is not None:
            default_args['app_version'] = app_version
        if is_docker():
            default_args['docker'] = 'True'

        payload = {
            "client_id": client_id,
            "non_personalized_ads": False,
            "events": [
                {
                    "name": event_action,
                    "params": {
                        "event_category": event_category,
                        "event_label": event_label,
                        "event_count": event_value,
                        "session_id": self.session_id,
                        **default_args,
                        **self.stats
                    }
                }
            ]
        }
        return payload

    def build_session_start_message(self, category: str, **kwargs):
        self.generate_new_session_id()
        return self.build_event_message(category, "session", "start", 1)

    def build_session_end_message(self, category: str, **kwargs):
        return self.build_event_message(category, "session", "end", 1)

    def build_error_message(self, category: str, error_msg: str, **kwargs):
        return self.build_event_message(category, "error_", error_msg, 1)

    def build_stack_trace_message(self, category: str, error_msg: str, **kwargs):
        return self.build_event_message(category, "stack_trace", error_msg, 1)

    def generate_new_cid_file(self):
        self.cid = get_or_generate_cid(self.cid_filename, lambda: str(uuid.uuid4()), is_valid_cid,
                                       self.old_cid_filename)

    def cid_file_initialized(self):
        return self.cid is not None

    def generate_new_session_id(self):
        self.session_id = str(uuid.uuid4())

    def remove_cid_file(self):
        self.cid = None
        remove_cid_file(self.cid_filename)
        remove_cid_file(self.old_cid_filename)

    def set_stats(self, data: dict):
        self.stats = data


def is_valid_cid(cid: str):
    try:
        uuid.UUID(cid, version=4)
    except ValueError:
        return False
    return True



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import sys
from platform import system


def print_without_end_line(string: str):
    """
    Prints a string without end line.
    :param string: a string that will be printed on the screen.
    """
    sys.stdout.write(string)
    sys.stdout.flush()


def input_with_timeout(prompt: str, timeout: int):
    """
    Requests user input and waits until the timeout expired.
    :param prompt: a string that will be printed on the screen.
    :param timeout: timeout to wait.
    :return: input string or empty line if the timeout expired.
    """
    platform = system()
    res_str = ''
    if platform == 'Windows':
        import msvcrt
        import time

        print_without_end_line(prompt)
        start_time = time.monotonic()
        end_time = start_time + timeout
        sleep_time = 0.05

        while time.monotonic() < end_time:
            if msvcrt.kbhit():
                c = msvcrt.getwche()
                if c in ['\r', '\n']:
                    print()
                    return res_str
                if c == '\003':
                    raise KeyboardInterrupt
                if c == '\b':
                    res_str = res_str[:-1]
                    print_without_end_line(''.join(['\r', ' ' * len(prompt + res_str + ' '), '\r', prompt, res_str]))
                else:
                    res_str += c
            time.sleep(sleep_time)

        print()
        return ''
    else:
        import selectors

        print_without_end_line(prompt)
        with selectors.DefaultSelector() as selector:
            selector.register(sys.stdin, selectors.EVENT_READ)
            events = selector.select(timeout)

            if events:
                key, _ = events[0]
                res_str = key.fileobj.readline().rstrip('\n')
            else:
                print()
            selector.unregister(sys.stdin)
        return res_str



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import abc

from ..utils.message import Message


class BackendRegistry:
    """
    The class that stores information about all registered telemetry backends
    """
    r = {}

    @classmethod
    def register_backend(cls, id: str, backend):
        cls.r[id] = backend

    @classmethod
    def get_backend(cls, id: str):
        if id not in cls.r:
            raise RuntimeError('The backend with id "{}" is not registered'.format(id))
        return cls.r.get(id)


class TelemetryBackendMetaClass(abc.ABCMeta):
    def __init__(cls, clsname, bases, methods):
        super().__init__(clsname, bases, methods)
        if cls.id is not None:
            BackendRegistry.register_backend(cls.id, cls)


class TelemetryBackend(metaclass=TelemetryBackendMetaClass):
    id = None

    @abc.abstractmethod
    def __init__(self, tid: str, app_name: str, app_version: str):
        """
        Initializer of the class
        :param tid: database id
        :param app_name: name of the application
        :param app_version: version of the application
        """

    @abc.abstractmethod
    def send(self, message: Message):
        """
        Sends the message to the backend.
        :param message: The Message object to send
        :return: None
        """

    @abc.abstractmethod
    def build_event_message(self, event_category: str, event_action: str, event_label: str, event_value: int = 1,
                            **kwargs):
        """
        Should return the Message object build from the event message.
        """

    @abc.abstractmethod
    def build_error_message(self, category: str, error_msg: str, **kwargs):
        """
        Should return the Message object build from the error message.
        """

    @abc.abstractmethod
    def build_stack_trace_message(self, category: str, error_msg: str, **kwargs):
        """
        Should return the Message object build from the stack trace message.
        """

    @abc.abstractmethod
    def build_session_start_message(self, category: str, **kwargs):
        """
        Should return the Message object corresponding to the session start.
        """

    @abc.abstractmethod
    def build_session_end_message(self, category: str, **kwargs):
        """
        Should return the Message object corresponding to the session end.
        """

    @abc.abstractmethod
    def remove_cid_file(self):
        """
        Should remove client ID file.
        """

    @abc.abstractmethod
    def generate_new_cid_file(self):
        """
        Should generate new Client ID file.
        """

    @abc.abstractmethod
    def cid_file_initialized(self):
        """
        Should check if client ID file is initialized.

        :return: True if client ID file is initialized, otherwise False.
        """

    @abc.abstractmethod
    def set_stats(self, data: dict):
        """
        Pass additional statistics, which will be added to telemetry messages
        """


# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from platform import system


def colored_print(text: str):
    platform = system()
    if platform == 'Windows':
        from ctypes import windll, Structure, byref
        from ctypes import wintypes

        class CONSOLE_SCREEN_BUFFER_INFO(Structure):
            _fields_ = [
                ("dwSize", wintypes._COORD),
                ("dwCursorPosition", wintypes._COORD),
                ("wAttributes", wintypes.WORD),
                ("srWindow", wintypes.SMALL_RECT),
                ("dwMaximumWindowSize", wintypes._COORD),
            ]

        console_screen_buffer_info = CONSOLE_SCREEN_BUFFER_INFO()
        windll.kernel32.GetConsoleScreenBufferInfo(windll.kernel32.GetStdHandle(-11), byref(console_screen_buffer_info))
        windll.kernel32.SetConsoleTextAttribute(windll.kernel32.GetStdHandle(-11), 0x0002)
        print(text)
        windll.kernel32.SetConsoleTextAttribute(windll.kernel32.GetStdHandle(-11),
                                                console_screen_buffer_info.wAttributes)
    else:
        print('\033[32m' + text + '\033[0m')



#!/usr/bin/env python3

# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import argparse
import openvino_telemetry as tm


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--opt_in", default=False, action='store_true',
                        help="Enabling sending anonymous telemetry data.")
    parser.add_argument("--opt_out", default=False, action='store_true',
                        help="Disabling sending anonymous telemetry data.")

    args = parser.parse_args()

    if args.opt_in is args.opt_out:
        print('Specify either "--opt_in" or "--opt_out" command line parameter.')
        exit(1)

    tid = "G-W5E9RNLD4H"
    if args.opt_in:
        tm.Telemetry.opt_in(tid=tid)
    else:
        tm.Telemetry.opt_out(tid=tid)
    exit(0)


if __name__ == "__main__":
    main()



# Copyright (C) 2018-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import logging as log
import os
import time
from enum import Enum
from pathlib import Path
from platform import system
from sys import stdin

from .colored_print import colored_print
from .input_with_timeout import input_with_timeout


class ConsentCheckResult(Enum):
    DECLINED = 0
    ACCEPTED = 1
    NO_FILE = 2


class DialogResult(Enum):
    DECLINED = 0
    ACCEPTED = 1
    TIMEOUT_REACHED = 2


class OptInChecker:
    dialog_timeout = 50  # seconds
    path_to_opt_in_out_script = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
    opt_in_out_script_name = "opt_in_out"
    doc_link = "docs.openvino.ai"
    opt_in_out_script_run_command = "\'{} --opt_out\'".format(opt_in_out_script_name)
    opt_in_question = "Intel would like your permission to collect software performance and usage data for the " \
                      "purpose of improving Intel products and services. This data will be collected directly " \
                      "by Intel or through the use of Google Analytics. This data will be stored in countries " \
                      "where Intel or Google operate. Intel operates around the world and your usage data will " \
                      "be sent to Intel in the United States or other countries.\nYou can opt-out at any time " \
                      "in the future by running {}.\n" \
                      "More Information is available at {}.\n" \
                      "Please type ‘Y’ to give your consent or ‘N’ to decline.".format(opt_in_out_script_run_command,
                                                                                       doc_link)
    opt_in_question_incorrect_input = "Please type ‘Y’ to give your consent or ‘N’ to decline."
    response_confirmation_accept = "The selected option was to collect telemetry data."
    response_confirmation_decline = "The selected option was NOT to collect telemetry data."
    response_confirmation_timer_reached = "The timer has expired and no data will be collected."

    @staticmethod
    def _ask_opt_in(question: str, timeout: int):
        """
        Runs input with timeout and checks user input.
        :param question: question that will be printed on the screen.
        :param timeout: timeout to wait.
        :return: opt-in dialog result.
        """
        colored_print(question)
        answer = input_with_timeout(prompt='>>', timeout=timeout)
        answer = answer.lower().strip()
        if answer == "n" or answer == "no":
            colored_print(OptInChecker.response_confirmation_decline)
            return DialogResult.DECLINED
        if answer == "y" or answer == "yes":
            colored_print(OptInChecker.response_confirmation_accept)
            return DialogResult.ACCEPTED
        return DialogResult.TIMEOUT_REACHED

    def opt_in_dialog(self):
        """
        Runs opt-in dialog until the timeout is expired.
        :return: opt-in dialog result.
        """
        start_time = time.time()
        answer = self._ask_opt_in(self.opt_in_question, self.dialog_timeout)
        time_passed = time.time() - start_time
        while time_passed < self.dialog_timeout and answer == DialogResult.TIMEOUT_REACHED:
            answer = self._ask_opt_in(self.opt_in_question_incorrect_input, self.dialog_timeout - time_passed)
            time_passed = time.time() - start_time

        if answer == DialogResult.TIMEOUT_REACHED:
            colored_print(OptInChecker.response_confirmation_timer_reached)

        return answer

    @staticmethod
    def consent_file_base_dir():
        """
        Returns the base directory of the consent file.
        :return: base directory of the consent file.
        """
        platform = system()

        dir_to_check = None

        if platform == 'Windows':
            dir_to_check = '$LOCALAPPDATA'
        elif platform in ['Linux', 'Darwin']:
            dir_to_check = Path.home()

        if dir_to_check is None:
            log.info('Failed to find location of the openvino_telemetry file.')
            return None

        consent_base_dir = os.path.expandvars(dir_to_check)
        if not os.path.isdir(consent_base_dir):
            log.info('Failed to find location of the openvino_telemetry file.')
            return None

        return consent_base_dir

    @staticmethod
    def consent_file_subdirectory():
        """
        Returns consent file subdirectory.
        :return: consent file subdirectory.
        """
        platform = system()
        if platform == 'Windows':
            return 'Intel Corporation'
        elif platform in ['Linux', 'Darwin']:
            return 'intel'
        log.info('Failed to find location of the openvino_telemetry file.')
        return None

    def consent_file(self):
        """
        Returns the consent file path.
        :return: consent file path.
        """
        return os.path.join(self.consent_file_base_dir(), self.consent_file_subdirectory(), "openvino_telemetry")

    def create_new_consent_file(self):
        """
        Creates a new consent file.
        :return: True if the file is created successfully, otherwise False
        """
        if not self.create_or_check_consent_dir():
            return False
        try:
            open(self.consent_file(), 'w').close()
        except Exception:
            return False
        return True

    def create_or_check_consent_dir(self):
        """
        Creates consent file directory and checks if the directory is writable.
        :return: True if the directory is created and writable, otherwise False
        """
        base_dir = self.consent_file_base_dir()
        consent_file_subdirectory = self.consent_file_subdirectory()
        if base_dir is None or consent_file_subdirectory is None:
            return False
        base_is_dir = os.path.isdir(base_dir)
        base_dir_exists = os.path.exists(base_dir)
        base_w_access = os.access(base_dir, os.W_OK)

        if not base_dir_exists or not base_is_dir:
            return False
        if not base_w_access:
            log.warning("Failed to create openvino_telemetry file. "
                        "Please allow write access to the following directory: {}".format(base_dir))
            return False

        consent_file_dir = os.path.join(base_dir, consent_file_subdirectory)
        consent_file_is_dir = os.path.isdir(consent_file_dir)
        consent_file_dir_exists = os.path.exists(consent_file_dir)

        # If consent path exists and it is not directory, we try to remove it
        if consent_file_dir_exists and not consent_file_is_dir:
            try:
                os.remove(consent_file_dir)
            except:
                log.warning("Unable to create directory for openvino_telemetry file, as {} is invalid directory.".format(consent_file_dir))
                return False

        if not os.path.exists(consent_file_dir):
            try:
                os.mkdir(consent_file_dir)

                # check that directory is created
                if not os.path.exists(consent_file_dir):
                    return False
            except Exception as e:
                log.warning("Failed to create directory for openvino_telemetry file: {}".format(str(e)))
                return False

        consent_file_w_access = os.access(consent_file_dir, os.W_OK)
        if not consent_file_w_access:
            log.warning("Failed to create openvino_telemetry file. "
                        "Please allow write access to the following directory: {}".format(consent_file_dir))
            return False
        return True

    def update_result(self, result: ConsentCheckResult):
        """
        Updates the 'opt_in' value in the consent file.
        :param result: opt-in dialog result.
        :return: False if the consent file is not writable, otherwise True
        """
        if self.consent_file_base_dir() is None or self.consent_file_subdirectory() is None:
            return False
        if not os.path.exists(self.consent_file()):
            if not self.create_new_consent_file():
                return False
        if not os.access(self.consent_file(), os.W_OK):
            log.warning("Failed to update opt-in status. "
                        "Please allow write access to the following file: {}".format(self.consent_file()))
            return False
        try:
            with open(self.consent_file(), 'w') as file:
                if result == ConsentCheckResult.ACCEPTED:
                    file.write("1")
                else:
                    file.write("0")
        except Exception:
            return False
        return True

    def consent_file_is_empty(self):
        """
        Checks if the consent file is empty.
        :return: True if consent file is empty, otherwise False.
        """
        if os.stat(self.consent_file()).st_size == 0:
            return True
        return False

    def get_info_from_consent_file(self):
        """
        Gets information from consent file.
        :return: the tuple, where the first element is True if the file is read successfully, otherwise False
        and the second element is the content of the consent file.
        """
        if not os.access(self.consent_file(), os.R_OK):
            return False, {}
        try:
            with open(self.consent_file(), 'r') as file:
                content = file.readline().strip()
        except Exception:
            return False, {}
        return True, content

    @staticmethod
    def _check_input_is_terminal():
        """
        Checks if stdin is terminal.
        :return: True if stdin is terminal, otherwise False
        """
        return stdin.isatty()

    @staticmethod
    def _check_main_process():
        platform = system()
        if platform == 'Windows':
            # In Windows 'os' module does not have getpid() and getsid(),
            # so the following checks are not applicable.
            # Subprocess check in Windows is handled by self._check_input_is_terminal(),
            # which does not work for Unix subprocesses.
            return True

        try:
            # Check that current process is the leader of process group
            if os.getpid() != os.getpgid(0):
                return False

            # Check that parent process is in same session as current process
            if os.getsid(os.getppid()) != os.getsid(0):
                return False
        except:
            # If we couldn't check main process, disable opt-in dialog
            return False
        return True

    @staticmethod
    def _check_run_in_notebook():
        """
        Checks that script is executed in Jupyter Notebook.
        :return: True script is executed in Jupyter Notebook, otherwise False
        """
        try:
            return get_ipython().__class__.__name__ == 'ZMQInteractiveShell'
        except NameError:
            pass
        return False

    @staticmethod
    def _run_in_ci():
        """
        Checks that script is executed in CI job.
        :return: True if script is executed in CI job, otherwise False
        """
        if "CI" in os.environ and os.environ["CI"].lower() == "true":
            return True

        if "TF_BUILD" in os.environ and len(os.environ["TF_BUILD"]):
            return True

        if "JENKINS_URL" in os.environ and len(os.environ["JENKINS_URL"]):
            return True

        return False

    def check(self, enable_opt_in_dialog, disable_in_ci=False):
        """
        Checks if user has accepted the collection of the information by checking the consent file.
        :return: consent check result
        """
        if self.consent_file_base_dir() is None or self.consent_file_subdirectory() is None:
            return ConsentCheckResult.DECLINED

        if disable_in_ci and self._run_in_ci():
            return ConsentCheckResult.DECLINED

        if not os.path.exists(self.consent_file()):
            if enable_opt_in_dialog:
                if not self._check_main_process():
                    return ConsentCheckResult.DECLINED

                if not self._check_input_is_terminal() or self._check_run_in_notebook():
                    return ConsentCheckResult.DECLINED
            return ConsentCheckResult.NO_FILE

        if not self.consent_file_is_empty():
            _, content = self.get_info_from_consent_file()
            if content == "1":
                return ConsentCheckResult.ACCEPTED
            elif content == "0":
                return ConsentCheckResult.DECLINED
        log.warning("Incorrect format of the file with opt-in status.")
        return ConsentCheckResult.DECLINED
